import unittest

from pyparsing import ParseException

from pylispng import grammar

simpleOperation = "(+ 1 2)"

simpleLambda = "(lambda (x) (* y z))"

simpleFunction = """(def length
   (lambda (x)
      (cond
         ((not x) 0)
         (   t   (+ 1 (length (cdr x))))
      )
   )
)
"""

class GrammarTestCase(unittest.TestCase):
    """
    """
    def test_emptyString(self):
        self.assertRaises(ParseException, grammar.sexp.parseString, "")

    def test_simpleOperation(self):
        sexpr = grammar.sexp.parseString(simpleOperation)
        topLevelList = sexpr.asList()
        self.assertEquals(len(topLevelList), 1)
        secondLevelList = sexpr.asList()[0]
        self.assertEquals(len(secondLevelList), 3)
        functionName = secondLevelList[0]
        self.assertEquals(functionName, "+")

    def test_simpleLambda(self):
        sexpr = grammar.sexp.parseString(simpleLambda)
        topLevelList = sexpr.asList()
        self.assertEquals(len(topLevelList), 1)
        secondLevelList = sexpr.asList()[0]
        self.assertEquals(len(secondLevelList), 3)
        functionName = secondLevelList[0]
        self.assertEquals(functionName, "lambda")
        parameters = secondLevelList[1]
        self.assertEquals(parameters[0], "x")
        body = secondLevelList[2]
        self.assertTrue(isinstance(body, list))
        self.assertEquals(body[0], "*")
        self.assertEquals(body[1], "y")
        self.assertEquals(body[2], "z")

    def test_simpleFunction(self):
        sexpr = grammar.sexp.parseString(simpleFunction)
        topLevelList = sexpr.asList()
        self.assertEquals(len(topLevelList), 1)
        secondLevelList = sexpr.asList()[0]
        self.assertEquals(len(secondLevelList), 3)
        self.assertEquals(secondLevelList[:2], ["def", "length"])
        body = secondLevelList[2]
        self.assertEquals(body[2][2][1][2][1], ["cdr", "x"])


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(GrammarTestCase),
        ))


def run_suite(verbosity=1):
    runner = unittest.TextTestRunner(verbosity=verbosity)
    runner.run(test_suite())

if __name__ == '__main__':
    run_suite()
