import unittest

from pyparsing import ParseException

from pylispng.parser import NewParser
from pylispng.test import test_grammar 


class NewParserTestCase(unittest.TestCase):
    """
    """
    def test_emptyString(self):
        p = NewParser()
        self.assertEquals(p.getLength(), 0)
        self.assertEquals(p.getStringCount(), 0)
        self.assertEquals(p.getSExpr(), None)

    def test_simpleOperation(self):
        p = NewParser(test_grammar.simpleOperation)
        self.assertEquals(p.getLength(), 3)
        self.assertEquals(p.getStringCount(), 7)
        sexpr = p.getSExpr()
        self.assertEquals([x.value for x in sexpr[1:]], [1, 2])
        self.assertEquals(sexpr[0].name, "+")

    def test_simpleLambda(self):
        p = NewParser(test_grammar.simpleLambda)
        self.assertEquals(p.getLength(), 3)
        self.assertEquals(p.getStringCount(), 20)
        sexpr = p.getSExpr()
        self.assertEquals(sexpr[0].name, "lambda")
        # The "x" is in a list because it was wrapped in parens in the
        # original string.
        self.assertEquals(sexpr[1], ["x"])
        subSexpr = sexpr[2]
        self.assertEquals(subSexpr[1:], ["y", "z"])
        self.assertEquals(subSexpr[0].name, "*")

    def test_simpleFunction(self):
        p = NewParser(test_grammar.simpleFunction)
        self.assertEquals(p.getLength(), 3)
        self.assertEquals(p.getStringCount(), 116)
        sexpr = p.getSExpr()
        self.assertEquals(sexpr[0].name, "def")
        self.assertEquals(sexpr[1], "length")
        subSexpr1 = sexpr[2]
        subSexpr2 = subSexpr1[2]
        self.assertEquals(subSexpr2[0].name, "cond")
        subSexpr3 = subSexpr2[1][0]
        self.assertEquals(subSexpr3[0].name, "not")
        subSexpr4 = subSexpr2[2]
        self.assertEquals(subSexpr4[0], "t")
        subSexpr5 = subSexpr4[1]
        self.assertEquals(subSexpr5[0].name, "+")
        subSexpr6 = subSexpr5[2]
        self.assertEquals(subSexpr6[0], "length")
        subSexpr7 = subSexpr6[1]
        self.assertEquals(subSexpr7[0].name, "cdr")
        self.assertEquals(subSexpr7[1], "x")
        """
        sexpr = grammar.sexp.parseString(simpleFunction)
        topLevelList = sexpr.asList()
        self.assertEquals(len(topLevelList), 1)
        secondLevelList = sexpr.asList()[0]
        self.assertEquals(len(secondLevelList), 3)
        self.assertEquals(secondLevelList[:2], ["def", "length"])
        body = secondLevelList[2]
        self.assertEquals(body[2][2][1][2][1], ["cdr", "x"])
        """


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(NewParserTestCase),
        ))


def run_suite(verbosity=1):
    runner = unittest.TextTestRunner(verbosity=verbosity)
    runner.run(test_suite())

if __name__ == '__main__':
    run_suite()
