import unittest

from pylispng import sexpr
from pylispng.util import DynamicList


class SExpressionsTestCase(unittest.TestCase):
    """

    """
    def setUp(self):
        self.exp1 = ('(+ 2 2)')
        self.exp2 = ('(+ 2 (- 5 2))')
        self.exp3 = ('(+ (+ (+ 0 0) (- 0 2)) (+ (+ 0 2) (- 1 0)))')
        self.exp4 = ('''(+ (+ (+ 0 0) (- 0 2)) (+ (+ (+ (+ (+ 0 0)
            (- 0 2)) (+ (+ 0 2) (- 1 0))) 2) (- 1 0)))''')
        self.exp5 = '(- 5 (* 2 4))'
        self.exp6 = ('(/ 1 4)')
        self.exp7 = ('(+ 2 (- 5 (/ 1 4)))')
        self.exps = [self.exp1, self.exp2, self.exp3, self.exp4,
            self.exp5, self.exp6, self.exp7]

    def test_expressionSize(self):
        """

        """
        exp2 = sexpr.SExpression(self.exp2)
        exp3 = sexpr.SExpression(self.exp3)
        exp4 = sexpr.SExpression(self.exp4)
        self.assertEquals(exp2.getSize(), 2)
        self.assertEquals(exp3.getSize(), 7)
        self.assertEquals(exp4.getSize(), 14)

    def test_expressionIndices(self):
        """

        """
        exp1 = sexpr.SExpression(self.exp1)
        exp2 = sexpr.SExpression(self.exp2)
        exp3 = sexpr.SExpression(self.exp3)
        #self.assertEquals(exp1[0], '(+ 2 2)')
        #self.assertEquals(exp2[1], ['-', 5, 2])
        #for i in xrange(exp3.getSize()):
        #    self.assertEquals(exp3[i], exp3Answer[i])
        # XXX hrm... need to think of a better way of representing/storing
        # branches such that manipulating them at a lower level will be
        # immediately reflected in the whole.
        #exp3[2] = ['*', 4, 3]

    def test_expressionSlices(self):
        """

        """
        exp3 = sexpr.SExpression(self.exp3)
        #self.assertEquals(
        #    exp3[1:3],
        #    [['+', ['+', 0, 0], ['-', 0, 2]],
        #        ['+', ['+', 0, 2], ['-', 1, 0]]])
        #self.assertEquals(exp3[5:7], [['+', 0, 2], ['-', 1, 0]])
        #self.assertEquals(exp3[4:], [['-', 0, 2], ['+', 0, 2], ['-', 1, 0]])

    def test_getExpressionDepth(self):
        """

        """
        for expected, exp in zip([1, 2, 3, 6, 2, 1, 3], self.exps):
            expr = sexpr.SExpression(exp)
            self.assertEquals(expr.getDepth(), expected)

    def test_getOperatorCount(self):
        """

        """
        answers = [1, 2, 7, 14, 2, 1, 3]
        initCount = 0
        for expected, exp in zip(answers, self.exps):
            expr = sexpr.SExpression(exp)
            self.assertEquals(expr.getSize(), expected)

    def test_getSubExpression(self):
        """

        """
        exp4 = sexpr.SExpression(self.exp4)
        #for index in xrange(14):
        #    self.assertEquals(exp4[index], answers[index])

    def test_evalExpressions(self):
        """

        """
        answers = [4, 5, 1, 2, -3, 0.25, 6.75]
        for expected, exp in zip(answers, self.exps):
            self.assertEquals(sexpr.SExpression(exp).eval(), expected)

    def test_appendStrings(self):
        e = sexpr.SExpression()
        e.append('+')
        e.append('1')
        e.append('2')
        self.assertEquals(str(e), '(+ 1 2)')
        self.assertEquals(e.eval(), 3)

    def test_nestedAppend(self):
        e1 = sexpr.SExpression()
        e2 = sexpr.SExpression()

        e2.append('-')
        e2.append('7')
        e2.append('2')

        e1.append('+')
        e1.append('1')
        e1.append(e2)
        self.assertEquals(str(e1), '(+ 1 (- 7 2))')
        self.assertEquals(e1.eval(), 6)

    def test_deeperNestedAppend(self):
        e1 = sexpr.SExpression()
        e2 = sexpr.SExpression()
        e3 = sexpr.SExpression()

        e3.append('-')
        e3.append('7')
        e3.append('2')

        e2.append('*')
        e2.append(e3)
        e2.append('3')

        e1.append('+')
        e1.append('1')
        e1.append(e2)
        self.assertEquals(str(e1), '(+ 1 (* (- 7 2) 3))')
        self.assertEquals(e1.eval(), 16)

    def test_expressionGraphNodes(self):
        """
        """

    def test_expressionGraphEdges(self):
        """
        """

    def test_expressionGraphEdges(self):
        """
        """

def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(SExpressionsTestCase),
        ))

def run_suite(verbosity=1):
    runner = unittest.TextTestRunner(verbosity=verbosity)
    runner.run(test_suite())

if __name__ == '__main__':
    run_suite()
