import unittest

from pylispng import parser
from pylispng import sexpr


ans0 = """
   [+]
    |--[2]
    |--[6]
"""

ans1 = """
   [*]
    |--[2]
    +--[*]
        +--[*]
            |--[3]
            |--[4]
        +--[*]
            |--[2]
            |--[3]
"""

ans2 = """
   [*]
    |--[5]
    +--[+]
        |--[3]
        |--[4]
        |--[6]
    |--[22]
    +--[*]
        |--[2.4]
        |--[5]
        |--[0.001]
    +--[-]
        |--[5]
        +--[+]
            |--[2]
            +--[-]
                |--[2]
                |--[4]
"""


ans3 = """
   [+]
    |--[1]
    +--[+]
        |--[2]
        +--[+]
            |--[3]
            |--[4]
"""


class SExpressionPrintTestCase(unittest.TestCase):
    """

    """
    def setUp(self):
        self.exprs = [
            '(+ 2 6)',
            '(* 2 (* (* 3 4) (* 2 3)))',
            '(* 5 (+ 3 4 6) 22 (* 2.4 5 0.001) (- 5 (+ 2 (- 2 4))))',
            '(+ 1 (+ 2 (+ 3 4)))',
            ]

    def test_ParserSExpression(self):
        """

        """
        # drop the first new line character, added above to be more readable
        answers = [x.lstrip() for x in [ans0, ans1, ans2, ans3]]
        p = parser.Parser()
        for expr, expected in zip(self.exprs, answers):
            s = p.getSExpr(expr)
            result = sexpr.getTree(s).lstrip()
            self.assertEquals(result, expected)

    def test_SExpression(self):
        s = sexpr.SExpression(self.exprs[2])
        self.assertEquals(sexpr.getTree(s).lstrip(), ans2.lstrip())

    def test_Graph(self):
        s = sexpr.SExpression(self.exprs[2])
        graph = s.getGraph()
        self.assertEquals(sexpr.getTree(graph).lstrip(), ans2.lstrip())


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(SExpressionPrintTestCase),
        ))


def run_suite(verbosity=1):
    runner = unittest.TextTestRunner(verbosity=verbosity)
    runner.run(test_suite())

if __name__ == '__main__':
    run_suite()       
