"""
The implementation below is from GvR's 1998 paper, updated in 2000 (see
http://www.python.org/doc/essays/graphs.html).

The original implementation was a series of functions with the graph simply
being a dict. That was converted into an object for this example.
"""

class Graph(dict):
    """
    >>> graph_data = {'A': ['B', 'C'],
    ...          'B': ['C', 'D'],
    ...          'C': ['D'],
    ...          'D': ['C'],
    ...          'E': ['F'],
    ...          'F': ['C']}

    >>> graph = Graph(graph_data)

    >>> graph.find_path('A', 'D')
    ['A', 'B', 'C', 'D']

    >>> graph.find_all_paths('A', 'D')
    [['A', 'B', 'C', 'D'], ['A', 'B', 'D'], ['A', 'C', 'D']]

    >>> graph.find_shortest_path('A', 'D')
    ['A', 'B', 'D']
    """
    def find_path(self, start, end, path=[]):
        path = path + [start]
        if start == end:
            return path
        if not self.has_key(start):
            return None
        for node in self[start]:
            if node not in path:
                newpath = self.find_path(node, end, path)
                if newpath:
                     return newpath
        return None

    def find_all_paths(self, start, end, path=[]):
        path = path + [start]
        if start == end:
            return [path]
        if not self.has_key(start):
            return []
        paths = []
        for node in self[start]:
            if node not in path:
                newpaths = self.find_all_paths(node, end, path)
                for newpath in newpaths:
                    paths.append(newpath)
        return paths

    def find_shortest_path(self, start, end, path=[]):
        path = path + [start]
        if start == end:
            return path
        if not self.has_key(start):
            return None
        shortest = None
        for node in self[start]:
            if node not in path:
                newpath = self.find_shortest_path(node, end, path)
                if newpath:
                    if not shortest or len(newpath) < len(shortest):
                        shortest = newpath
        return shortest


if __name__ == "__main__":
    import doctest
    doctest.testmod()
